<?php
/**
 * @package       Joomla.Plugin
 * @subpackage    Editors.switcher
 * @version       4.1.1
 *
 * @copyright     (C) 2007 Yoshiki Kozaki
 * @license       https://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 *
 * @author        Yoshiki Kozaki
 * @link          https://forgetaz.com/
 *
 */


namespace Joomla\Plugin\Editors\Switcher\Extension;

use Joomla\CMS\Factory;
use Joomla\CMS\Application\CMSApplicationInterface;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\Registry\Registry;
use Joomla\CMS\Editor\Editor;

// no direct access
\defined('_JEXEC') or die;

/**
 * Switcher plugin.
 *
 * @package   switcher
 * @since     4.0.0
 */
final class Switcher extends CMSPlugin
{
	/**
	 * Application object
	 *
	 * @var    ?Joomla\CMS\Application\CMSApplication
	 * @since  4.0.0
	 */
	protected ?CMSApplicationInterface $app = null;

	/**
	 * Affects constructor behavior. If true, language files will be loaded automatically.
	 *
	 * @var    boolean
	 * @since  3.1.4
	 */
	protected $autoloadLanguage = true;

	/**
	 * Editor object
	 *
	 * @var Joomla\CMS\Plugin\CMSPlugin
	 * @since 4.0.0
	 */
	protected object $switchereditor;

	/**
	 * Cookie name
	 *
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected string $cookiename = 'editorswitchercurrent';

	/**
	 * Constructor
	 *
	 * @param   object  $subject  The object to observe
	 * @param   array   $config   An array that holds the plugin configuration
	 *
	 * @throws Exception
	 * @since       1.5
	 */
	public function __construct(&$subject, $config)
	{
		parent::__construct($subject, $config);
//		var_dump(PluginHelper::getPlugin('editors', 'switcher')->params);exit;
		$reg = new Registry(PluginHelper::getPlugin('editors', 'switcher')->params);
		$editor = $this->app->getInput()->cookie->get($this->cookiename,
			$reg->get('default_editor', 'none'));

		if (PluginHelper::isEnabled('editors', $editor))
		{
			$plugin = PluginHelper::getPlugin('editors', $editor);
			$this->setSwitcherEditor($plugin->name);
		}
		else
		{
			$this->app->enqueueMessage(
				Text::_('PLG_EDITOR_SWITCHER_EDITORWASNOTFOUND'), 'error');
			//change to editors - none
			$plugin = PluginHelper::getPlugin('editors', 'none');
			$this->setSwitcherEditor($plugin->name);
		}
	}

	/**
	 * Create the selected editor
	 *
	 * @param   object  $subject
	 * @param   array   $config
	 * @param   object  $plugin
	 *
	 * @since 1.0.0
	 */
	protected function setSwitcherEditor(string $editor):void
	{
		$this->switchereditor = $this->app->bootPlugin($editor, 'editors');
	}

	/**
	 * Create the selector of editors
	 *
	 * @staticvar null $selector
	 *
	 * @param   string  $current
	 *
	 * @return string
	 * @since     1.0.0
	 */
	protected function getEditorSelector(string $current): string
	{
		//TODO for Multiple Editors
		static $selector = '';

		if ($selector == '')
		{
			$params  = new Registry(PluginHelper::getPlugin('editors', 'switcher')->params);
			$editors = PluginHelper::getPlugin('editors');

			//Search Index of current editor
			$count = 0;
			$index = 0;
			$array = array();
			foreach ($editors as $k => $o)
			{
				if ($o->name == 'switcher')
				{
					unset($editors[$k]);
					continue;
				}

				$array[$o->name] = $count;
				$o->text         = ucfirst($o->name);
				if ($o->name == $current)
				{
					$index = $count;
				}

				$count++;
			}

			$confirmation = $params->get('confirmation', 1);

			//onchange
			$js = 'onchange="';
			if ($confirmation)
			{
				$this->loadLanguage('plg_editors_switcher', JPATH_ADMINISTRATOR);
				$js .= 'if(this.options.selectedIndex != ' . $index . ' && confirm(\''
					. Text::_('PLG_EDITORS_SWITCHER_CONFIRM_MESSAGE_TITLE', true)
					. '\r\n' . Text::_('PLG_EDITORS_SWITCHER_CONFIRM_MESSAGE', true) . '\')'
					. '){';
				$js .= 'document.getElementById(\'editorswitcher-currentvalue\').value = this.options.selectedIndex;';
			}

			$js .= "document.cookie = '$this->cookiename = '+this.value+'; samesite=lax'; window.location.reload();";

			if ($confirmation)
			{
				$js .= '}';
			}
			$js .= '"';

			//html
			$selector = '<div id="switcherSelector" style="vertical-align:middle;margin:0 0 0 5px;display:inline-block;">'
				. '<input type="hidden" id="editorswitcher-currentvalue" value="'. $current . '" />'
				. HTMLHelper::_('select.genericlist', $editors, 'switcheditor'
					, 'class="form-select" ' . $js, 'name', 'text', $current, 'jswitcheditor')
				. '</div>';

			// Write html and move to wrapper of buttons
			$arrayJson = json_encode($array);
			$js        = "var jswitcherEditors = $arrayJson;document.addEventListener('DOMContentLoaded',"
				. "function(){var btnwrap = document.getElementsByClassName('editor-xtd-buttons');if(btnwrap.length > 0)"
				. " btnwrap[0].appendChild(document.getElementById('switcherSelector'));});";
			$this->app->getDocument()->getWebAssetManager()->addInlineScript($js);
		}

		return $selector;
	}

	/**
	 * Initialises the Editor.
	 *
	 * @return  void
	 *
	 * @since 1.5
	 */
	public function onInit():void
	{
		if (is_callable(array($this->switchereditor, 'onInit')))
		{
			$this->switchereditor->onInit();
		}
	}


	/**
	 * Display the editor area.
	 *
	 * @param   string   $name     The control name.
	 * @param   string   $content  The contents of the text area.
	 * @param   string   $width    The width of the text area (px or %).
	 * @param   string   $height   The height of the text area (px or %).
	 * @param   int      $col      The number of columns for the textarea.
	 * @param   int      $row      The number of rows for the textarea.
	 * @param   boolean  $buttons  True and the editor buttons will be displayed.
	 * @param   string   $id       An optional ID for the textarea (note: since 1.6). If not supplied the name is used.
	 * @param   string   $asset    Not used.
	 * @param   object   $author   Not used.
	 * @param   array    $params   Associative array of editor parameters.
	 *
	 * @return  string  HTML
	 * @since 1.5
	 */
	public function onDisplay($name, $content, $width, $height, $col, $row, $buttons = true, $id = null, $asset = null, $author = null, $params = array()): string
	{
		if (is_callable(array($this->switchereditor, 'onDisplay')))
		{
			//Display the specified editor and EditorSelector
			return $this->switchereditor->onDisplay($name, $content, $width, $height, $col, $row, $buttons, $id, $asset, $author, $params)
				. $this->getEditorSelector($this->switchereditor->_name);
		}

		return '';
	}
}
